"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var W3_1 = require("./W3");
// tslint:disable:max-line-length
var soltsice;
(function (soltsice) {
    // tslint:disable-next-line:typedef
    function parseArgs(args) {
        if (args.paths.length < 1 || args.paths.length > 3) {
            throw new Error('Wrong number of args');
        }
        if (args.cleanArtifacts) {
            throw new Error('Cleaning artifacts is not supported yet');
        }
        if (args.skipPattern) {
            throw new Error('Skip pattern is not supported yet');
        }
        var options = {
            source: args.paths[0],
            destination: args.paths.length > 1 ? args.paths[1] : args.paths[0],
            W3importPath: args.paths.length > 2 ? args.paths[2] : 'soltsice',
            cleanArtifacts: args.cleanArtifacts,
            skipPattern: args.skipPattern
        };
        console.log('Soltsice optoins: ', options);
        return options;
    }
    // tslint:disable-next-line:typedef
    function generateTypes(args) {
        var options = parseArgs(args);
        var endOfLine = require('os').EOL;
        var destination = options.destination;
        if (!fs.existsSync(destination)) {
            fs.mkdirSync(destination);
        }
        var files = getSourceFiles(options.source);
        // console.log('FILES', files);
        var paths = getSourcePaths(options.source, files);
        // console.log('PATHS', paths);
        var targets = getDestinationPaths(options.destination, files);
        // console.log('TARGETS', targets);
        paths.forEach(function (file, idx) {
            var ts = processFile(files[idx], file, targets[idx], options.W3importPath);
            console.log('Writing to ', targets[idx]);
            fs.writeFileSync(targets[idx], ts);
        });
        // create index file
        var exports = files.map(function (f) { return "export * from './" + f.replace('.json', '') + "';"; }).join(endOfLine);
        fs.writeFileSync(path.join(destination, 'index.ts'), exports);
    }
    soltsice.generateTypes = generateTypes;
    function getSourceFiles(src) {
        var filesArr = fs.readdirSync(src).filter(function (f) { return f.endsWith('.json'); });
        return filesArr;
    }
    function getSourcePaths(src, files) {
        var filesArr = files.map(function (f) { return path.resolve(path.join(src, f)); }); // __dirname,
        filesArr.forEach(function (file) {
            // console.log(file);
        });
        return filesArr;
    }
    function getDestinationPaths(destination, files) {
        var filesArr = files
            .map(function (f) { return path.resolve(path.join(destination, f)); }) // __dirname,
            .map(function (f) { return f.replace('.json', '.ts'); });
        return filesArr;
    }
    function abiTypeToTypeName(abiType, isReturnType) {
        var outputType;
        var arrayPosition = -1;
        if (!abiType) {
            outputType = ['void'];
        }
        else {
            arrayPosition = abiType.indexOf('[');
            if (arrayPosition >= 0) {
                abiType = abiType.substring(0, arrayPosition);
            }
            if (abiType.startsWith('uint') || abiType.startsWith('int')) {
                outputType = isReturnType ? ['BigNumber'] : ['BigNumber', 'number'];
            }
            else if (abiType.startsWith('bytes')) {
                outputType = ['string'];
            }
            else {
                //     export type ABIDataTypes = 'uint256' | 'boolean' | 'string' | 'bytes' | string; // TODO complete list
                switch (abiType) {
                    case 'bool':
                        outputType = ['boolean'];
                        break;
                    case 'string':
                    case 'address':
                        outputType = ['string'];
                        break;
                    default:
                        console.warn('Not implemented ABI type, using `any` instead: ', abiType);
                        outputType = ['any'];
                }
            }
        }
        if (arrayPosition >= 0) {
            outputType = outputType.map(function (x) { return x + '[]'; });
        }
        return outputType.join(' | ');
    }
    function processCtor(abi) {
        var inputs = abi.inputs;
        var inputsString;
        var inputsNamesString;
        if (inputs && inputs.length > 0) {
            inputsString = inputs.map(function (i) { return i.name + ': ' + abiTypeToTypeName(i.type); }).join(', ');
            inputsNamesString = inputs.map(function (i) { return 'ctorParams!.' + i.name; }).join(', ');
        }
        else {
            inputsString = '';
            inputsNamesString = '';
        }
        return { typesNames: inputsString, names: inputsNamesString };
    }
    function processInputs(abi) {
        var inputs = abi.inputs;
        var inputsString;
        var inputsNamesString;
        if (inputs && inputs.length > 0) {
            inputs = inputs.map(function (i, idx) { return (i.name === '' ? Object.assign(i, { name: '_' + idx }) : i); });
            inputsString = inputs.map(function (i) { return i.name + ': ' + abiTypeToTypeName(i.type); }).join(', '); // comma for tx params
            inputsNamesString = inputs.map(function (i) { return i.name; }).join(', ');
        }
        else {
            inputsString = '';
            inputsNamesString = '';
        }
        return { typesNames: inputsString, names: inputsNamesString };
    }
    function processAbi(abi) {
        var name = abi.name;
        var isConstant = abi.constant;
        var inputs = processInputs(abi);
        var inputsString = inputs.typesNames;
        var inputsNamesString = inputs.names;
        var outputs = abi.outputs;
        var outputType;
        if (outputs && outputs.length > 1) {
            console.warn('Multiple output ABI not implemented, using `any` type for: ', abi);
            outputType = 'any';
        }
        else {
            outputType = abiTypeToTypeName(outputs && outputs.length > 0 ? outputs[0].type : undefined, true);
        }
        var methodsBody = isConstant
            ? "\n  // tslint:disable-next-line:max-line-length\n  // tslint:disable-next-line:variable-name\n  public " + name + "(" + (inputsString === '' ? '' : inputsString + ',') + " txParams?: W3.TX.TxParams): Promise<" + outputType + "> {\n    return new Promise((resolve, reject) => {\n      this._instance." + name + "\n        .call(" + (inputsNamesString === '' ? '' : inputsNamesString + ',') + " txParams || this._sendParams)\n        .then((res: any) => resolve(res))\n        .catch((err: any) => reject(err));\n    });\n  }\n  "
            : "\n  // tslint:disable-next-line:member-ordering\n  public " + name + " = Object.assign(\n      // tslint:disable-next-line:max-line-length\n      // tslint:disable-next-line:variable-name\n      (" + (inputsString === '' ? '' : inputsString + ',') + " txParams?: W3.TX.TxParams, privateKey?: string): Promise<W3.TX.TransactionResult> => {\n          if (!privateKey) {\n            return new Promise((resolve, reject) => {\n              this._instance." + name + "(" + (inputsNamesString === '' ? '' : inputsNamesString + ',') + " txParams || this._sendParams)\n                .then((res: any) => resolve(res))\n                .catch((err: any) => reject(err));\n            });\n          } else {\n            // tslint:disable-next-line:max-line-length\n            return this.w3.sendSignedTransaction(this.address, privateKey, this._instance." + name + ".request(" + inputsNamesString + ").params[0].data, txParams || this._sendParams, undefined)\n              .then(txHash => {\n                return this.waitTransactionReceipt(txHash);\n              });\n          }\n      },\n      {\n        // tslint:disable-next-line:max-line-length\n        // tslint:disable-next-line:variable-name\n        sendTransaction: Object.assign((" + (inputsString === '' ? '' : inputsString + ',') + " txParams?: W3.TX.TxParams): Promise<string> => {\n              return new Promise((resolve, reject) => {\n                this._instance." + name + ".sendTransaction(" + (inputsNamesString === '' ? '' : inputsNamesString + ',') + " txParams || this._sendParams)\n                  .then((res: any) => resolve(res))\n                  .catch((err: any) => reject(err));\n              });\n            },\n            {\n              // tslint:disable-next-line:max-line-length\n              // tslint:disable-next-line:variable-name\n              sendSigned: (" + (inputsString === '' ? '' : inputsString + ',') + " privateKey: string, txParams?: W3.TX.TxParams, nonce?: number): Promise<string> => {\n                // tslint:disable-next-line:max-line-length\n                return this.w3.sendSignedTransaction(this.address, privateKey, this._instance." + name + ".request(" + inputsNamesString + ").params[0].data, txParams || this._sendParams, nonce);\n              }\n            }\n        )\n      },\n      {\n        // tslint:disable-next-line:max-line-length\n        // tslint:disable-next-line:variable-name\n        data: (" + inputsString + "): Promise<string> => {\n          return new Promise((resolve, reject) => {\n            resolve(this._instance." + name + ".request(" + inputsNamesString + ").params[0].data);\n          });\n        }\n      },\n      {\n        // tslint:disable-next-line:max-line-length\n        // tslint:disable-next-line:variable-name\n        estimateGas: (" + inputsString + "): Promise<number> => {\n          return new Promise((resolve, reject) => {\n            this._instance." + name + ".estimateGas(" + inputsNamesString + ").then((g: any) => resolve(g)).catch((err: any) => reject(err));\n          });\n        }\n      });\n  ";
        return methodsBody;
    }
    function processFile(fileName, filePath, targetPath, W3ImportPath) {
        console.log('Processing ', filePath);
        var contract = JSON.parse(fs.readFileSync(filePath, { encoding: 'utf8' }));
        var importPath = W3ImportPath;
        var artifactRelPath = path.relative(path.dirname(targetPath), filePath).replace(/\\/g, '/'); // path.resolve(filePath).replace(/\\/g, '/'); //
        console.log('REL PATH ', artifactRelPath);
        var contractName = contract.contract_name || contract.contractName;
        if (!contractName) {
            throw new Error('Cannot find contract name in the artifact');
        }
        var abis = contract.abi;
        // When a contract is created, its constructor (a function declared with the constructor keyword)
        // is executed once. A constructor is optional. Only one constructor is allowed, and this means
        // overloading is not supported.
        var ctor = abis.filter(function (a) { return a.type === 'constructor'; });
        var ctorParams = ctor.length === 1 ? processCtor(ctor[0]) : { typesNames: '', names: '' };
        var names = abis.map(function (x) { return x.name; });
        var methodsBody = abis
            .filter(function (value, index, self) {
            // filter duplicate names and keep only the first one
            // order of filters matters because we use names arrays built from abis before the next 'function' filter
            return names.indexOf(value.name) === index;
        })
            .filter(function (a) { return a.type === 'function'; })
            .map(processAbi)
            .join('');
        var bnImport = "";
        if (ctorParams.typesNames.indexOf('BigNumber') >= 0 || methodsBody.indexOf('BigNumber') >= 0) {
            bnImport = "\n    import { BigNumber } from 'bignumber.js';";
        }
        // TODO
        var template = bnImport + "\nimport { W3, SoltsiceContract } from '" + importPath + "';\n\n/**\n * " + contractName + " API\n */\nexport class " + contractName + " extends SoltsiceContract {\n\n  public static get artifacts() { return require('" + artifactRelPath + "'); }\n\n  public static get bytecodeHash() {\n    // we need this before ctor, but artifacts are static and we cannot pass it to the base class, so need to generate\n    let artifacts = " + contractName + ".artifacts;\n    if (!artifacts || !artifacts.bytecode) {\n        return undefined;\n    }\n    let hash = W3.sha3(JSON.stringify(artifacts.bytecode));\n    return hash;\n  }\n\n  // tslint:disable-next-line:max-line-length\n  public static async new(deploymentParams: W3.TX.TxParams, ctorParams?: {" + ctorParams.typesNames + "}, w3?: W3, link?: SoltsiceContract[], privateKey?: string): Promise<" + contractName + "> {\n    w3 = w3 || W3.default;\n    if (!privateKey) {\n      let contract = new " + contractName + "(deploymentParams, ctorParams, w3, link);\n      await contract._instancePromise;\n      return contract;\n    } else {\n      let data = " + contractName + ".newData(ctorParams, w3);\n      let txHash = await w3.sendSignedTransaction(W3.zeroAddress, privateKey, data, deploymentParams);\n      let txReceipt = await w3.waitTransactionReceipt(txHash);\n      let rawAddress = txReceipt.contractAddress!;\n      let contract = await " + contractName + ".at(rawAddress, w3);\n      return contract;\n    }\n  }\n\n  public static async at(address: string | object, w3?: W3): Promise<" + contractName + "> {\n    let contract = new " + contractName + "(address, undefined, w3, undefined);\n    await contract._instancePromise;\n    return contract;\n  }\n\n  public static async deployed(w3?: W3): Promise<" + contractName + "> {\n    let contract = new " + contractName + "('', undefined, w3, undefined);\n    await contract._instancePromise;\n    return contract;\n  }\n\n  // tslint:disable-next-line:max-line-length\n  public static newData(ctorParams?: {" + ctorParams.typesNames + "}, w3?: W3): string {\n    // tslint:disable-next-line:max-line-length\n    let data = SoltsiceContract.newDataImpl(w3, " + contractName + ".artifacts, ctorParams ? [" + ctorParams.names + "] : []);\n    return data;\n  }\n\n  protected constructor(\n    deploymentParams: string | W3.TX.TxParams | object,\n    ctorParams?: {" + ctorParams.typesNames + "},\n    w3?: W3,\n    link?: SoltsiceContract[]\n  ) {\n      // tslint:disable-next-line:max-line-length\n      super(\n        w3,\n        " + contractName + ".artifacts,\n        ctorParams ? [" + ctorParams.names + "] : [],\n        deploymentParams,\n        link\n      );\n  }\n  /*\n    Contract methods\n  */\n  " + methodsBody + "\n}\n";
        return template;
    }
    /** Create or get local key file and return private key and address. This is a blocking sync function for file read/write, therefore should be used during initial startup. */
    function getLocalPrivateKeyAndAddress(filepath, password) {
        var address = '';
        var privateKey = '';
        var publicKey = '';
        var keythereum = W3_1.W3.getKeythereum();
        var createNew = function () {
            var dk = keythereum.create();
            privateKey = W3_1.W3.EthUtils.bufferToHex(dk.privateKey);
            publicKey = W3_1.W3.EthUtils.bufferToHex(W3_1.W3.EthUtils.privateToPublic(dk.privateKey));
            var addrBuffer = W3_1.W3.EthUtils.privateToAddress(dk.privateKey);
            address = W3_1.W3.EthUtils.bufferToHex(addrBuffer);
            var keyObject = keythereum.dump(password, dk.privateKey, dk.salt, dk.iv);
            fs.writeFileSync(filepath, JSON.stringify(keyObject), { encoding: 'ascii' });
        };
        if (fs.existsSync(filepath)) {
            var keyObject = JSON.parse(fs.readFileSync(filepath, { encoding: 'ascii' }).trim());
            address = keyObject.address;
            var privateBuffer = keythereum.recover(password, keyObject);
            privateKey = W3_1.W3.EthUtils.bufferToHex(privateBuffer);
            publicKey = W3_1.W3.EthUtils.bufferToHex(W3_1.W3.EthUtils.privateToPublic(privateBuffer));
        }
        else {
            createNew();
        }
        if (privateKey && !privateKey.startsWith('0x')) {
            privateKey = '0x' + privateKey;
        }
        if (publicKey && !publicKey.startsWith('0x')) {
            publicKey = '0x' + publicKey;
        }
        if (address && !address.startsWith('0x')) {
            address = '0x' + address;
        }
        return { privateKey: privateKey, publicKey: publicKey, address: address };
    }
    soltsice.getLocalPrivateKeyAndAddress = getLocalPrivateKeyAndAddress;
})(soltsice = exports.soltsice || (exports.soltsice = {}));
//# sourceMappingURL=soltsice.js.map